#include "M3x3.h"
#include <math.h>
#include <iostream>

V3& M3x3::operator[](int i) {

  return rows[i];

}

V3 M3x3::operator*(V3 &v) {

  V3 ret;
  ret[0] = rows[0]*v;
  ret[1] = rows[1]*v;
  ret[2] = rows[2]*v;

  return ret;

}

void M3x3::SetColumn(int i, V3 v) {

  rows[0][i] = v[0];
  rows[1][i] = v[1];
  rows[2][i] = v[2];

}

V3 M3x3::GetColumn(int i) {

  V3 ret;
  ret[0] = rows[0][i];
  ret[1] = rows[1][i];
  ret[2] = rows[2][i];
  return ret;

}

M3x3 M3x3::operator *(M3x3 &m) {

  M3x3 ret;
  ret.SetColumn(0, (*this)*m.GetColumn(0));
  ret.SetColumn(1, (*this)*m.GetColumn(1));
  ret.SetColumn(2, (*this)*m.GetColumn(2));
  return ret;

}

M3x3 M3x3::Invert() {

  M3x3 ret;
  V3 a = GetColumn(0), b = GetColumn(1), c = GetColumn(2);
  V3 _a = b ^ c; _a = _a / (a * _a);
  V3 _b = c ^ a; _b = _b / (b * _b);
  V3 _c = a ^ b; _c = _c / (c * _c);
  ret[0] = _a;
  ret[1] = _b;
  ret[2] = _c;

  return ret;

}

ostream& operator<<(ostream& ostr, M3x3& m) {

  ostr << m[0] << endl << m[1] << endl << m[2] << endl;
  return ostr;

}

M3x3::M3x3(char* matType, float theta) {


  float _theta = theta * 3.1415f / 180.0f;

  if (!strcmp(matType, _MATS3x3_H_ROT_X)) {
    rows[0][0] = 1.0f;
    rows[0][1] = 0;
    rows[0][2] = 0;
    rows[1][0] = 0;
    rows[1][1] = cosf(_theta);
    rows[1][2] = -sinf(_theta);
    rows[2][0] = 0;
    rows[2][1] = sinf(_theta);
    rows[2][2] = cosf(_theta);
    return;
  }
  else if (!strcmp(matType, _MATS3x3_H_ROT_Y)) {
    rows[0][0] = cosf(_theta);
    rows[0][1] = 0;
    rows[0][2] = sinf(_theta);
    rows[1][0] = 0;
    rows[1][1] = 1;
    rows[1][2] = 0;
    rows[2][0] = -sinf(_theta);
    rows[2][1] = 0;
    rows[2][2] = cosf(_theta);
    return;
  }
  // add the other rotations as needed
  cerr << "ERROR: unsupported matrix type" << endl;

}

void M3x3::Transpose() {

  M3x3 m = *this;
  SetColumn(0, m[0]);
  SetColumn(1, m[1]);
  SetColumn(2, m[2]);

}