#include "scene.h"
#include "v3.h"
#include "M3x3.h"

#include <iostream>


using namespace std;

Scene *scene = 0;

Scene::Scene() {

  tstep = 10.0f;
  moveProj = false;
  gui = new GUI;
  fb = new FrameBuffer(20, 40, 480, 320);
  fb->label("SW Frame buffer");
  hwfb = new FrameBuffer(20, 40+40+fb->h, 480, 320);
  hwfb->label("HW Frame buffer");
  hwfb->isHW = true;
  gui->show();
  tmesh = new TMesh();
  tmesh->Load("geometry/teapot1K.bin");
  V3 newCenter(0.0f, 0.0f, -50.0f);
  float newDiag = 40.0f;
  tmesh->PlaceAndScale(newCenter, newDiag);
  V3 center(0.0f, 0.0f, -50.0f);
  float dims[2] = {30.0f, 30.0f};
#if 0
  tmesh = new TMesh(center, dims);
  V3 colors[4];
//  unsigned int cols[4] = {0xFF0000FF, 0xFF00FF00, 0xFFFF0000, 0xFF00FFFF};
  unsigned int cols[4] = {0xFF000000, 0xFF0000FF, 0xFF0000FF, 0xFF000000};
  for (int i = 0; i < 4; i++)
    colors[i].SetFromUIntColor(cols[i]);
  tmesh->SetColors(colors);
  tmesh->SetTCsQuad();
#endif
  int tw = 512;
  FrameBuffer *texture = new FrameBuffer(20, fb->h+40+40, tw, tw);
  int checkerSize = 32;
  unsigned int checkerColors[2] = {0xFF000000, 0xFFFFFFFF};
  texture->SetGrid(checkerSize, checkerColors);
  texture->label("texture");
  bool useProj = false;
  projFB = 0;
  if (useProj) {
    projFB = new FrameBuffer(10, 10, 320, 240);
    checkerColors[0] = 0xFFFF00FF; 
    checkerColors[1] = 0xFF00FF00; 
    projFB->show();
    projFB->SetGrid(40, checkerColors);
  }
//  if (texture->w <= 512)
//    texture->show();
//  tmesh->texture = texture;
  Show();
  ppc = new PPC(45.0f, fb->w, fb->h, 10.0f, 100.0f);
  if (useProj) {
    projPPC = new PPC(15.0f, projFB->w, projFB->h, 10.0f, 100.0f);
    projPPC->C = V3(0.0f, 0.0f, -10.0f);
    projPPC->UpdateInvCam();
  }

  cgi = new CGInterface();
  soi = new ShaderOneInterface();

  Redraw();

}

void Scene::Show() {

  gui->uiw->position(20+fb->w+20, 40);
  unsigned int color = 0xFF000000;
  fb->Set(color);
  fb->show();
  hwfb->show();

}

void Scene::DBG() {

  {
    moveProj = !moveProj;
    return;
  }

  {
    V3 newC(50.0f, 25.0f, -50.0f);
    V3 newVD(-1.0f, -.45f, 0.0f);
    V3 quasiUp(0.0f, 1.0f, 0.0f);
    PPC ppc2 = *ppc;
    ppc2.PositionAndOrient(newC, newVD, quasiUp);
    ppc2.Visualize(ppc, fb);
    fb->redraw();
    return;
  }

  {
    V3 newC(50.0f, 25.0f, -50.0f);
    V3 newVD(-1.0f, -.45f, 0.0f);
    V3 quasiUp(0.0f, 1.0f, 0.0f);
    ppc->PositionAndOrient(newC, newVD, quasiUp);
    Redraw();
    return;
  }

  {
    TMesh tmesh;
    tmesh.Load("geometry/teapot1K.bin");
    return;
  }

  {

    M3x3 m;
    m[0] = V3(13.0f, -12.0f, 3.0f);
    m[1] = V3(-4.0f, 3.0f, 2.0f);
    m[2] = V3(5.0f, -12.0f, -10.0f);
    cerr << m << m.Invert() << m.Invert()*m << m*m.Invert();
    return;

  }

  {
    V3 v0(1.0f, 0.3f, 2.0f);
    V3 v1(1.0f, 0.3f, 2.0f);
    V3 v2(0.0f, 0.0f, 0.0f);
    v2 = v0+v1;
    cerr << v2 << endl;
    cerr << v2[0] << endl;
    v2[1] = 34.0f;
    cerr << v2[1] << endl;
    M3x3 m;
    m[0] = v0;
    m[1] = v1;
    m[2] = v2;

  }

  {
    unsigned int colors[2] = {0xFFAAAAAA, 0xFFFFFFFF};
    fb->SetGrid(40, colors);
    fb->redraw();
    return;
  }

  {
    int u, v = 50, sizePix = 15;
    unsigned int color = 0xFFFFFFFF;
    for (u = 0; u < fb->w; u++) {
      fb->Set(0xFF000000);
      fb->DrawDiagonalCross(u, v, color, sizePix);
      fb->redraw();
      Fl::check();
    }
  }

}

void Scene::TranslateForward() {

  PPC *useppc = ppc;
  if (moveProj) {
    useppc = projPPC;
  }
  if (!useppc)
    return;
  useppc->Translate(ppc->GetVD()*tstep);
  Redraw();

}
  
void Scene::TranslateBackward() {

  PPC *useppc = ppc;
  if (moveProj) {
    useppc = projPPC;
  }
  if (!useppc)
    return;
  useppc->Translate(ppc->GetVD()*-tstep);
  Redraw();

}
  
void Scene::TranslateRight() {

  PPC *useppc = ppc;
  if (moveProj) {
    useppc = projPPC;
  }
  if (!useppc)
    return;
  useppc->Translate(ppc->a.UnitVector()*tstep);
  Redraw();

}
  
void Scene::TranslateLeft() {

  PPC *useppc = ppc;
  if (moveProj) {
    useppc = projPPC;
  }
  if (!useppc)
    return;
  useppc->Translate(ppc->a.UnitVector()*-tstep);
  Redraw();

}
  
void Scene::TranslateUp() {

  PPC *useppc = ppc;
  if (moveProj) {
    useppc = projPPC;
  }
  if (!useppc)
    return;
  useppc->Translate(ppc->b.UnitVector()*-tstep);
  Redraw();

}

void Scene::TranslateDown() {

  PPC *useppc = ppc;
  if (moveProj) {
    useppc = projPPC;
  }
  if (!useppc)
    return;
  useppc->Translate(ppc->b.UnitVector()*tstep);
  Redraw();

}

void Scene::ZoomIn() {

  float frel= 1.1f;
  if (!ppc)
    return;
  ppc->Zoom(frel);
  Redraw();

}

void Scene::Redraw() {

  if (fb) {
    // clear buffers
    unsigned int bgr = 0xFFFF0000;
    fb->Set(bgr);
    fb->SetDepth(0.0f);

    // render geometry
    //  tmesh->RenderPoints(ppc, fb);
    tmesh->Render(ppc, fb, 0, 0);

    // post buffer
    fb->redraw();
  }

  if (hwfb) {
    hwfb->redraw();
  }

}

void Scene::RenderHWGPU() {

  if (cgi->needInit) {
    cgi->PerSessionInit();
    soi->PerSessionInit(cgi);
  }

  // clear buffers
  glEnable(GL_DEPTH_TEST);
  glClearColor(0.0f, 0.0f, 1.0f, 1.0f);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // set view
  ppc->SetViewGL();

  // per frame parameter setting and enabling programs
  soi->PerFrameInit();
  soi->BindPrograms();
  cgi->EnableProfiles();

  // draw geometry
  tmesh->RenderHW();
  cgi->DisableProfiles();

}

void Scene::RenderHWFixedPipeline() {

  // clear buffers
  glEnable(GL_DEPTH_TEST);
  glClearColor(0.0f, 0.0f, 1.0f, 1.0f);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // set view
  ppc->SetViewGL();

  // draw geometry
  tmesh->RenderHW();

}